/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Peter Lunk (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.ui.handlers;

import com.google.common.collect.Iterables;
import java.io.File;
import java.util.Collection;
import java.util.List;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.emf.diffmerge.api.Role;
import org.eclipse.emf.diffmerge.api.diff.IDifference;
import org.eclipse.emf.diffmerge.diffdata.EComparison;
import org.eclipse.emf.diffmerge.patch.api.ModelPatch;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchException;
import org.eclipse.emf.diffmerge.patch.runtime.ModelPatchRecorder;
import org.eclipse.emf.diffmerge.patch.serializer.IModelPatchSerializer;
import org.eclipse.emf.diffmerge.patch.ui.handlers.ComparisonSelectionUtil;
import org.eclipse.emf.diffmerge.patch.ui.preferences.ModelPatchPreferenceProvider;
import org.eclipse.emf.diffmerge.patch.ui.utils.DialogFactory;
import org.eclipse.emf.diffmerge.patch.ui.utils.SerializerProvider;
import org.eclipse.emf.diffmerge.ui.diffuidata.ComparisonSelection;
import org.eclipse.emf.diffmerge.ui.setup.EMFDiffMergeEditorInput;
import org.eclipse.emf.diffmerge.ui.viewers.EMFDiffNode;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Extension;

@SuppressWarnings("all")
public class ModelpatchGenerationHandler extends AbstractHandler {
  public static final String MODELPATCH_GENERATION_ERROR_TITLE = "Model Patch Generation Error";
  
  @Extension
  private ModelPatchPreferenceProvider _modelPatchPreferenceProvider = ModelPatchPreferenceProvider.INSTANCE;
  
  @Extension
  private ComparisonSelectionUtil util = new ComparisonSelectionUtil();
  
  @Override
  public Object execute(final ExecutionEvent event) throws ExecutionException {
    final Shell shell = HandlerUtil.getActiveShell(event);
    final IWorkbench workbench = HandlerUtil.getActiveWorkbenchWindow(event).getWorkbench();
    IStructuredSelection selection = null;
    try {
      final ISelection activeMenuSelection = HandlerUtil.getActiveMenuSelection(event);
      if ((activeMenuSelection instanceof IStructuredSelection)) {
        selection = ((IStructuredSelection)activeMenuSelection);
      }
      if ((selection == null)) {
        final ISelection currentSelection = HandlerUtil.getCurrentSelection(event);
        if ((currentSelection instanceof IStructuredSelection)) {
          selection = ((IStructuredSelection)currentSelection);
        }
      }
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception ex = (Exception)_t;
        ex.printStackTrace();
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    final IEditorInput part = HandlerUtil.getActiveEditorInput(event);
    EMFDiffNode diffNode = null;
    if ((part instanceof EMFDiffMergeEditorInput)) {
      diffNode = ((EMFDiffMergeEditorInput)part).getCompareResult();
      final ISelection editorSelection = ((EMFDiffMergeEditorInput)part).getViewer().getSelection();
      if ((editorSelection instanceof IStructuredSelection)) {
        selection = ((IStructuredSelection)editorSelection);
      }
    }
    if ((selection instanceof ComparisonSelection)) {
      if ((diffNode == null)) {
        diffNode = ((ComparisonSelection)selection).getDiffNode();
      }
      final Collection<IDifference> diffs = this.util.getDiffsToMerge(((ComparisonSelection)selection), false, diffNode);
      boolean _isEmpty = diffs.isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        this.generatePatch(diffs, shell, workbench, selection);
      }
    } else {
      if (((selection != null) && (diffNode != null))) {
        final EComparison comparison = diffNode.getActualComparison();
        List<IDifference> _differences = comparison.getDifferences(Role.REFERENCE);
        List<IDifference> _differences_1 = comparison.getDifferences(Role.TARGET);
        final Iterable<IDifference> differences = Iterables.<IDifference>concat(_differences, _differences_1);
        this.generatePatch(differences, shell, workbench, selection);
      }
    }
    return null;
  }
  
  public void generatePatchFromDiffNode(final EMFDiffNode diffNode, final ComparisonSelection selection, final Shell shell, final IWorkbench workbench) {
    final Collection<IDifference> diffs = this.util.getDiffsToMerge(selection, false, diffNode);
    boolean _isEmpty = diffs.isEmpty();
    boolean _not = (!_isEmpty);
    if (_not) {
      this.generatePatch(diffs, shell, workbench, selection);
    }
  }
  
  public void generatePatch(final Iterable<IDifference> diff, final Shell shell, final IWorkbench workbench, final IStructuredSelection selection) {
    @Extension
    final DialogFactory factory = new DialogFactory(shell);
    final SerializerProvider serializerProvider = new SerializerProvider();
    try {
      final IModelPatchSerializer serializer = serializerProvider.getSelectedSerializer(this._modelPatchPreferenceProvider.getSerializationType());
      final IFile path = factory.openSaveFileDialog(workbench, selection, serializer.getPreferredFileExtension());
      if ((path != null)) {
        final ModelPatchRecorder modelPatchRecorder = new ModelPatchRecorder();
        final ModelPatch generatedPatch = modelPatchRecorder.generateModelPatch(diff);
        final File patchFile = path.getLocation().toFile();
        serializer.serialize(generatedPatch, patchFile);
        boolean _isFileGenerated = this.isFileGenerated(patchFile);
        boolean _equals = (_isFileGenerated == true);
        if (_equals) {
          factory.openModelPatchEditor(workbench.getActiveWorkbenchWindow(), path);
        } else {
          StringConcatenation _builder = new StringConcatenation();
          _builder.append("File cannot be created at the following path: ");
          _builder.append(path);
          factory.openErrorDialog(ModelpatchGenerationHandler.MODELPATCH_GENERATION_ERROR_TITLE, _builder.toString());
        }
      } else {
        factory.openInformationDialog("Model Patch Generation Information", "No file has been selected!");
      }
    } catch (final Throwable _t) {
      if (_t instanceof ModelPatchException) {
        final ModelPatchException ex = (ModelPatchException)_t;
        factory.openErrorDialog(ModelpatchGenerationHandler.MODELPATCH_GENERATION_ERROR_TITLE, "Modelpatch generation finished with errors!", ex, 
          "org.eclipse.emf.diffmerge.patch");
      } else if (_t instanceof ClassCastException) {
        factory.openErrorDialog(ModelpatchGenerationHandler.MODELPATCH_GENERATION_ERROR_TITLE, "Patch generation only works from EMF Diff/Merge!");
      } else if (_t instanceof Exception) {
        final Exception ex_2 = (Exception)_t;
        ex_2.printStackTrace();
        factory.openErrorDialog(ModelpatchGenerationHandler.MODELPATCH_GENERATION_ERROR_TITLE, "Unknown error!", ex_2, "org.eclipse.emf.diffmerge.patch");
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    return;
  }
  
  public boolean isFileGenerated(final File file) {
    return file.exists();
  }
}
